#include <dht.h>
#include <LiquidCrystal_I2C.h>

#define DHT_PIN 7

//Definining Termometer Icon
byte tempchar1[8]={B00000, B00001, B00010, B00100, 
                  B00100, B00100, B00100, B00111,};
byte tempchar2[8]={B00111, B00111, B00111, B01111, 
                  B11111, B11111, B01111, B00011,};
byte tempchar3[8]={B00000, B10000, B01011, B00100, 
                  B00111, B00100, B00111, B11100,};
byte tempchar4[8]={B11111, B11100, B11100, B11110, 
                  B11111, B11111, B11110, B11000,};

//Defining Humidity Icon
byte humchar1[8]={B00000, B00001, B00011, B00011, 
                  B00111, B01111, B01111, B11111,};
byte humchar2[8]={B11111, B11111, B11111, B01111, 
                  B00011, B00000, B00000, B00000,};
byte humchar3[8]={B00000, B10000, B11000, B11000, 
                  B11100, B11110, B11110, B11111,};
byte humchar4[8]={B11111, B11111, B11111, B11110,
                  B11100, B00000, B00000, B00000,};                  

dht DHT;

LiquidCrystal_I2C lcd(0x27, 16, 2);

char readData[20];


void printLCD(int temp, int humidity, int location) {
  lcd.clear();

  // Print temperature icon + value
  lcd.setCursor(0,0);
  lcd.write(byte(0));
  lcd.setCursor(0,1);
  lcd.write(byte(1));
  lcd.setCursor(1, 0);
  lcd.write(byte(2));
  lcd.setCursor(1, 1);
  lcd.write(byte(3));
  lcd.setCursor(2, 1);
  lcd.print(temp);
  lcd.print((char)223);
  lcd.print("C");

  // Print humidity icon + value
  lcd.setCursor(14,0);
  lcd.write(byte(4));
  lcd.setCursor(14,1);
  lcd.write(byte(5));
  lcd.setCursor(15, 0);
  lcd.write(byte(6));
  lcd.setCursor(15, 1);
  lcd.write(byte(7));
  lcd.setCursor(11, 1);
  lcd.print(humidity);
  lcd.print("%");

  // Print location
  if (location == 0) {
    lcd.setCursor(6, 0);
    lcd.print("Local");
  } else {
    lcd.setCursor(6, 0);
    lcd.print("City");
  }
}

// Print DHT (local) values
void printDHT() {
  // Get data from sensor
  int chk = DHT.read11(DHT_PIN);

  // Print data
  printLCD((int) DHT.temperature, (int) DHT.humidity, 0);
}

// Receive and print remote values
void printWiFi() {
  lcd.clear();
  lcd.home();
  lcd.print("Receiving Data");

  int idx = 0;
  char byte;
  int temp = 0;
  int hum = 0;

  // If data is available
  if (Serial.available()) {
    // Search for start of sequence
    while ((byte = Serial.read()) != '<') {}

    // Get rest of data
    while (Serial.available()) {
      readData[idx++] = byte;

      // Look for end of sequence
      if ((byte = Serial.read()) == '>') {
        readData[idx++] = byte;
        readData[idx] = '\0';
        break;
      }
    }

    // Parse data
    char* aux = strtok(readData, "<,>");
    float auxFloat = atof(aux);
    temp = (int) auxFloat;
    aux = strtok(NULL, "<,>");
    auxFloat = atof(aux);
    hum = (int) auxFloat;

    // Print data
    printLCD(temp, hum, 1);
  }
}

void setup(void) {
  Serial.begin(115200);

  lcd.init();
  
  // Create custom characters
  lcd.createChar(0,tempchar1);
  lcd.createChar(1,tempchar2);
  lcd.createChar(2,tempchar3);
  lcd.createChar(3,tempchar4);

  lcd.createChar(4,humchar1);
  lcd.createChar(5,humchar2);
  lcd.createChar(6,humchar3);
  lcd.createChar(7,humchar4);

  lcd.clear();         
  lcd.backlight();
}

// Loop between local and remote data with a 5 second delay 
void loop() {
  printDHT();
  delay(5000);
  printWiFi();
  delay(5000);
}
