#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include <Arduino_JSON.h>

// WiFi parameters to be configured
const char* ssid = "GalaxyNetwork"; // Write here your router's username
const char* password = "twnv3321"; // Write here your router's passward

const String endpoint = "http://api.openweathermap.org/data/2.5/weather?q=";

// OpenWeatherMap api key to be configured
const String apiKey = "8a919f79768776ffdec6b47bfaf94ee7";

// City and country code to be configured
const String city = "Bucharest";
const String country = "RO";

// Last time a request was sent + wait time between requests
unsigned long lastTime = 0;
unsigned long waitTime = 600000; // 1000 = 1s => 600000 = 10min

// Buffer for JSON data from HTTP request
String jsonBuffer;

// Recorded temperature and humidity
String temperature = "";
String humidity = "";

void connectToWiFi() {
  WiFi.begin(ssid, password);

  // Wait for Wifi connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
}

String httpGETRequest(const char* path) {
  WiFiClient client;
  HTTPClient http;

  // Begin connection
  http.begin(client, path);

  // Send HTTP POST request
  int httpResponseCode = http.GET();

  String payload = "{}"; 

  // Check if successful
  if (httpResponseCode > 0) {
    payload = http.getString();
  } else {
    // Manage Error
  }

  http.end();

  return payload;
}

// Send data sequence to connected device
void sendData() {
  String res = "<" + temperature + "," + humidity + ">";
  Serial.write(res.c_str(), res.length());
}

void setup(void) { 
  Serial.begin(115200);

  connectToWiFi();
}

void loop() {
  // If $waitTime time passed send HTTP request
  if ((millis() - lastTime > waitTime) || (lastTime == 0)) {
    // If device still has WiFi connection
    if (WiFi.status() == WL_CONNECTED) {
      // Example request path:
      // https://api.openweathermap.org/data/2.5/weather?q=Bucharest,RO&APPID=8a919f79768776ffdec6b47bfaf94ee7
      String requestPath = endpoint + city + "," + country + "&units=metric&APPID=" + apiKey;

      // Get data + parse it
      jsonBuffer = httpGETRequest(requestPath.c_str());
      JSONVar jsonData = JSON.parse(jsonBuffer);

      // Check if parsing failed
      if (JSON.typeof(jsonData) == "undefined") {
        return;
      }

      // Get temperature
      temperature = JSON.stringify(jsonData["main"]["temp"]);
      // Get humidity
      humidity = JSON.stringify(jsonData["main"]["humidity"]);;

      // Register current time
      lastTime = millis();
    } else {
      // WiFi Disconnected
      connectToWiFi();
    }
  }
  sendData();
}
